/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * WC24hServer.java - IRMP Server class
 *
 * Copyright (c) 2014-2015 Frank Meyer - frank(at)fli4l.de
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */

package com.example.wc24h;

import android.util.Log;
import android.util.Xml;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlSerializer;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

class WC24hSettings
{
    class DisplayColor
    {
        int     red = 0;
        int     green = 0;
        int     blue = 0;
        int     white = 0;
    }
    class Animation
    {
        String  name = "";
        int     deceleration = 0;                                   // 1 = fastest, 15 = slowest
        int     default_deceleration = 0;                           // 1 = fastest, 15 = slowest
        int     flags = 0;                                          // yet not used
    }
    class NightTime
    {
        int     minutes = 0;
        int     flags = 0;
    }
	String                  ipaddress = "192.168.10.119";
    String                  description = "";
    private int[]           numvars = new    int[MAX_NUM_VARIABLES];
    private String[]        strvars = new String[MAX_STR_VARIABLES];
    private DisplayColor[]  colors = new DisplayColor[2];
    Vector<String>          display_modes = new Vector<>();
    Vector<Animation>       display_animations = new Vector<>();
    Vector<Animation>       color_animations = new Vector<>();
    Vector<Animation>       ambilight_modes = new Vector<>();
    Vector<NightTime>       night_times = new Vector<>();

    // numeric variables index names
	static final int DISPLAY_USE_RGBW_NUM_VAR                       = 0;
	static final int EEPROM_IS_UP_NUM_VAR                           = 1;
	static final int RTC_IS_UP_NUM_VAR                              = 2;
	static final int DISPLAY_POWER_NUM_VAR                          = 3;
	static final int DISPLAY_MODE_NUM_VAR                           = 4;
	static final int MAX_DISPLAY_MODES_NUM_VAR 					    = 5;
	static final int DISPLAY_BRIGHTNESS_NUM_VAR 					= 6;
	static final int DISPLAY_FLAGS_NUM_VAR 						    = 7;
	static final int DISPLAY_AUTOMATIC_BRIGHTNESS_ACTIVE_NUM_VAR 	= 8;
	static final int DISPLAY_TEMPERATURE_INTERVAL_NUM_VAR           = 9;
	static final int ANIMATION_MODE_NUM_VAR                         = 10;
	static final int AMBILIGHT_MODE_NUM_VAR                         = 11;
	static final int AMBILIGHT_LEDS_NUM_VAR                         = 12;
	static final int AMBILIGHT_OFFSET_NUM_VAR                       = 13;
	static final int AMBILIGHT_BRIGHTNESS_NUM_VAR                   = 14;
	static final int COLOR_ANIMATION_MODE_NUM_VAR                   = 15;
	static final int LDR_RAW_VALUE_NUM_VAR                          = 16;
	static final int LDR_MIN_VALUE_NUM_VAR                          = 17;
	static final int LDR_MAX_VALUE_NUM_VAR                          = 18;
	static final int TIMEZONE_NUM_VAR                               = 19;
	static final int DS18XX_IS_UP_NUM_VAR                           = 20;
	static final int RTC_TEMP_INDEX_NUM_VAR                         = 21;
	static final int RTC_TEMP_CORRECTION_NUM_VAR                    = 22;
	static final int DS18XX_TEMP_INDEX_NUM_VAR                      = 23;
	static final int DS18XX_TEMP_CORRECTION_NUM_VAR                 = 24;
	static final int DISPLAY_HEART_INTERVAL_NUM_VAR                 = 25;
	static final int DISPLAY_XMAS_TREE_INTERVAL_NUM_VAR             = 26;
	static final int DISPLAY_DATE_INTERVAL_NUM_VAR                  = 27;
	static final int DISPLAY_FIREWORKS_INTERVAL_NUM_VAR             = 28;
	static final int HARDWARE_CONFIGURATION_NUM_VAR                 = 29;
	static final int DISPLAY_AMBILIGHT_POWER_NUM_VAR                = 30;
	static final int TICKER_DECELRATION_NUM_VAR                     = 31;
	static final int MAX_NUM_VARIABLES                              = 32; // must be the last member

    // string variables index names
    static final int TICKER_TEXT_STR_VAR                            = 0;
    static final int VERSION_STR_VAR                                = 1;
    static final int EEPROM_VERSION_STR_VAR                         = 2;
    static final int ESP8266_VERSION_STR_VAR                        = 3;
    static final int TIMESERVER_STR_VAR                             = 4;
    static final int WEATHER_APPID_STR_VAR                          = 5;
    static final int WEATHER_CITY_STR_VAR                           = 6;
    static final int WEATHER_LON_STR_VAR                            = 7;
    static final int WEATHER_LAT_STR_VAR                            = 8;
    static final int UPDATE_HOST_VAR                                = 9;
    static final int UPDATE_PATH_VAR                                = 10;
    static final int MAX_STR_VARIABLES                              = 11; // must be the last member

    // colors
    static final int MAX_COLOR_VARIABLES                            = 2;

    WC24hSettings()
	{
        ipaddress = "";// "192.168.10.119";
        description = "";
        setDefaults();
	}

	boolean isRGBW()
    {
        return numvars[DISPLAY_USE_RGBW_NUM_VAR] != 0;
    }

	int getNumVar(int idx)
    {
        if (idx < 0 || idx >= MAX_NUM_VARIABLES)
        {
            return 0;
        }
        return numvars[idx];
    }

    void setNumVar(int idx, int value)
    {
        if (idx < 0 || idx >= MAX_NUM_VARIABLES)
        {
            return;
        }
        numvars[idx] = value;
    }

    String getStrVar(int idx)
    {
        if (idx < 0 || idx >= MAX_NUM_VARIABLES)
        {
            return "";
        }
        return strvars[idx];
    }

    void setStrVar(int idx, String value)
    {
        if (idx < 0 || idx >= MAX_NUM_VARIABLES)
        {
            return;
        }
        strvars[idx] = value;
    }

    DisplayColor getColor(int idx)
    {
        if (idx < 0 || idx >= MAX_COLOR_VARIABLES)
        {
            return null;
        }
        return colors[idx];
    }

    void setColor(int idx, DisplayColor color)
    {
        if (idx < 0 || idx >= MAX_COLOR_VARIABLES)
        {
            return;
        }
        colors[idx] = color;
    }

    private void setDefaults()
    {
        // set defaults
        for (int i = 0; i < MAX_NUM_VARIABLES; i++)
        {
            setNumVar(i, 0);
        }
        for (int i = 0; i < MAX_STR_VARIABLES; i++)
        {
            setStrVar(i, "");
        }
        setNumVar(DISPLAY_BRIGHTNESS_NUM_VAR, 15);
        for (int i = 0; i < MAX_COLOR_VARIABLES; i++)
        {
            colors[i] = new DisplayColor();
            colors[i].red = 0;
            colors[i].green = 0;
            colors[i].blue = 0;
            colors[i].white = 0;
        }
        display_modes.clear();
        display_animations.clear();
        color_animations.clear();
        ambilight_modes.clear();
        night_times.clear();
    }

	int loadData (File path, String filename)
	{
		int			rtc = 0;

		try
		{
            setDefaults();
            ipaddress = "";
            description = "";
			/* Read XML file */
            XmlPullParser parser = Xml.newPullParser();
            parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
            File file = new File(path, filename);
            parser.setInput(new FileReader(file));
            loadData(parser);
		}
		catch (Exception e)
		{
			// Toast.makeText(getApplicationContext(), "XML Error: " + e, Toast.LENGTH_SHORT).show();
        	Log.e("WC24h", "exception in loadData() method", e);
		}

		return rtc;
	}

    void loadData (XmlPullParser parser)
    {
        int			rtc = 0;
        String      name = "";

        try
        {
            // set defaults
            setDefaults();

            // read all XML elements
            int eventType = parser.getEventType();
            while (eventType != XmlPullParser.END_DOCUMENT)
            {
                switch (eventType) {
                    case XmlPullParser.START_DOCUMENT:
                        break;
                    case XmlPullParser.START_TAG:
                        name = parser.getName();
                        String sidx = parser.getAttributeValue(null, "idx");
                        if ("ipaddress".equals(name)) {
                            ipaddress = parser.getAttributeValue(null, "value").trim();
                            break;
                        }
                        if ("description".equals(name)) {
                            description = parser.getAttributeValue(null, "value").trim();
                            break;
                        }
                        if (sidx == null)
                        {
                            break;
                        }
                        int idx;
                        try {
                            idx = Integer.parseInt(sidx);
                        } catch (Exception e) {
                            Log.e("WC24h", "XML error:", e);
                            break;
                        }
                        if (idx < 0)
                        {
                            break;
                        }
                        if ("dspcolor".equals(name)) {
                            if (idx >= MAX_COLOR_VARIABLES)
                            {
                                break;
                            }
                            try {
                                colors[idx].red = Integer.parseInt(parser.getAttributeValue(null, "red"));
                                colors[idx].green = Integer.parseInt(parser.getAttributeValue(null, "green"));
                                colors[idx].blue = Integer.parseInt(parser.getAttributeValue(null, "blue"));
                                colors[idx].white = Integer.parseInt(parser.getAttributeValue(null, "white"));
                            } catch (Exception e) {
                                Log.e("WC24h", "XML error:", e);
                            }
                            break;
                        }
                        if ("dispmode".equals(name)) {
                            String val = parser.getAttributeValue(null, "name");
                            if (val == null || "".equals(val))
                            {
                                break;
                            }
                            if (display_modes.size() <= idx)
                            {
                                display_modes.setSize(idx + 1);
                            }
                            display_modes.set(idx, val);
                            break;
                        }
                        if ("dispanim".equals(name)) {
                            Animation animation = getAnimationData(parser);
                            if (animation == null)
                            {
                                break;
                            }
                            if (display_animations.size() <= idx)
                            {
                                display_animations.setSize(idx + 1);
                            }
                            display_animations.set(idx, animation);
                            break;
                        }
                        if ("coloranim".equals(name)) {
                            Animation animation = getAnimationData(parser);
                            if (animation == null)
                            {
                                break;
                            }
                            if (color_animations.size() <= idx)
                            {
                                color_animations.setSize(idx + 1);
                            }
                            color_animations.set(idx, animation);
                            break;
                        }
                        if ("almode".equals(name)) {
                            Animation animation = getAnimationData(parser);
                            if (animation == null)
                            {
                                break;
                            }
                            if (ambilight_modes.size() <= idx)
                            {
                                ambilight_modes.setSize(idx + 1);
                            }
                            ambilight_modes.set(idx, animation);
                            break;
                        }
                        if ("nighttime".equals(name)) {
                            NightTime nightTime = new NightTime();
                            try {
                                nightTime.minutes = Integer.parseInt(parser.getAttributeValue(null, "minutes"));
                                nightTime.flags = Integer.parseInt(parser.getAttributeValue(null, "flags"));
                                if (night_times.size() <= idx)
                                {
                                    night_times.setSize(idx + 1);
                                }
                                night_times.set(idx, nightTime);
                            } catch (Exception e) {
                                Log.e("WC24h", "XML error:", e);
                            }
                            break;
                        }
                        //Log.w("WC24h", "start: " + name);
                        String val = parser.getAttributeValue(null, "value");
                        //Log.w("WC24h", "name: " + name + " attr: " + idx + "=" + val);
                        if (val == null)
                        {
                            break;
                        }
                        try {
                            if ("numvar".equals(name)) {
                                setNumVar(idx, Integer.parseInt(val));
                            } else if ("strvar".equals(name)) {
                                setStrVar(idx, val);

                            }
                        } catch (Exception e) {
                            Log.e("WC24h", "XML error:", e);
                        }
                        break;
                    case XmlPullParser.END_TAG:
                        break;
                }
                eventType = parser.next();
            }
            Log.w("WC24h", "dsp_mode: " + getNumVar(WC24hSettings.DISPLAY_MODE_NUM_VAR));
            Log.w("WC24h", "anm_mode: " + getNumVar(WC24hSettings.ANIMATION_MODE_NUM_VAR));
            Log.w("WC24h", "clr_mode: " + getNumVar(WC24hSettings.COLOR_ANIMATION_MODE_NUM_VAR));
        }
        catch (Exception e)
        {
            // Toast.makeText(getApplicationContext(), "XML Error: " + e, Toast.LENGTH_SHORT).show();
            Log.e("WC24h", "exception in loadData() method", e);
        }
    }

    boolean saveData (File path, String filename)
    {
		boolean		rtc = false;
        File		newxmlfile = new File(path, filename);

        try
        {
        	newxmlfile.createNewFile();
        }
        catch(IOException e)
        {
        	Log.e("WC24h", "exception in saveData() method", e);
            return false;
        }

        FileOutputStream fileos = null;       	

        try
        {
        	fileos = new FileOutputStream(newxmlfile);
        }
        catch(FileNotFoundException e)
        {
        	Log.e("WC24h", "cannot create FileOutputStream", e);
            return false;
        }

        XmlSerializer serializer = Xml.newSerializer();

        try
        {
			serializer.setOutput(fileos, "UTF-8");
			serializer.startDocument(null, true);
			serializer.setFeature("http://xmlpull.org/v1/doc/features.html#indent-output", true); 

			serializer.startTag(null, "WC24h");
			{
                serializer.startTag(null, "ipaddress");
                serializer.attribute(null, "value", ipaddress);
                serializer.endTag(null, "ipaddress");

                serializer.startTag(null, "description");
                serializer.attribute(null, "value", description);
                serializer.endTag(null, "description");

                for (int i = 0; i < MAX_NUM_VARIABLES; i++) {
                    serializer.startTag(null, "numvar");
                    serializer.attribute(null, "idx", Integer.toString(i));
                    serializer.attribute(null, "value", Integer.toString(numvars[i]));
                    serializer.endTag(null, "numvar");
                }

                for (int i = 0; i < MAX_STR_VARIABLES; i++) {
                    serializer.startTag(null, "strvar");
                    serializer.attribute(null, "idx", Integer.toString(i));
                    serializer.attribute(null, "value", strvars[i]);
                    serializer.endTag(null, "strvar");
                }

                for (int i = 0; i < MAX_COLOR_VARIABLES; i++) {
                    serializer.startTag(null, "dspcolorsvar");
                    serializer.attribute(null, "idx", Integer.toString(i));
                    serializer.attribute(null, "red", Integer.toString(colors[i].red));
                    serializer.attribute(null, "green", Integer.toString(colors[i].green));
                    serializer.attribute(null, "blue", Integer.toString(colors[i].blue));
                    serializer.attribute(null, "white", Integer.toString(colors[i].white));
                    serializer.endTag(null, "dspcolorsvar");
                }

                for (int i = 0; i < display_modes.size(); i++) {
                    serializer.startTag(null, "dspmode");
                    serializer.attribute(null, "idx", Integer.toString(i));
                    serializer.attribute(null, "name", display_modes.elementAt(i));
                    serializer.endTag(null, "dspmode");
                }

                for (int i = 0; i < display_animations.size(); i++) {
                    serializer.startTag(null, "dispanim");
                    serializer.attribute(null, "idx", Integer.toString(i));
                    serializer.attribute(null, "name", display_animations.elementAt(i).name);
                    serializer.attribute(null, "dcl", Integer.toString(display_animations.elementAt(i).deceleration));
                    serializer.attribute(null, "def_dcl", Integer.toString(display_animations.elementAt(i).default_deceleration));
                    serializer.attribute(null, "flags", Integer.toString(display_animations.elementAt(i).flags));
                    serializer.endTag(null, "dispanim");
                }

                for (int i = 0; i < color_animations.size(); i++) {
                    serializer.startTag(null, "coloranim");
                    serializer.attribute(null, "idx", Integer.toString(i));
                    serializer.attribute(null, "name", color_animations.elementAt(i).name);
                    serializer.attribute(null, "dcl", Integer.toString(color_animations.elementAt(i).deceleration));
                    serializer.attribute(null, "def_dcl", Integer.toString(color_animations.elementAt(i).default_deceleration));
                    serializer.attribute(null, "flags", Integer.toString(color_animations.elementAt(i).flags));
                    serializer.endTag(null, "coloranim");
                }

                for (int i = 0; i < ambilight_modes.size(); i++) {
                    serializer.startTag(null, "almode");
                    serializer.attribute(null, "idx", Integer.toString(i));
                    serializer.attribute(null, "name", ambilight_modes.elementAt(i).name);
                    serializer.attribute(null, "dcl", Integer.toString(ambilight_modes.elementAt(i).deceleration));
                    serializer.attribute(null, "def_dcl", Integer.toString(ambilight_modes.elementAt(i).default_deceleration));
                    serializer.attribute(null, "flags", Integer.toString(ambilight_modes.elementAt(i).flags));
                    serializer.endTag(null, "almode");
                }

                for (int i = 0; i < night_times.size(); i++) {
                    serializer.startTag(null, "nighttime");
                    serializer.attribute(null, "idx", Integer.toString(i));
                    serializer.attribute(null, "minutes", Integer.toString(night_times.elementAt(i).minutes));
                    serializer.attribute(null, "flags", Integer.toString(night_times.elementAt(i).flags));
                    serializer.endTag(null, "nighttime");
                }
            }
			serializer.endTag(null, "WC24h");
			serializer.endDocument();
			serializer.flush();
			fileos.close();
			rtc = true;
		}
        catch (Exception e)
        {
			Log.e("WC24h", "error occurred while creating xml file", e);
		}

        return rtc;
    }

    private Animation getAnimationData(XmlPullParser parser)
    {
        try
        {
            Animation animation = new Animation();
            animation.name = parser.getAttributeValue(null, "name");
            animation.deceleration = Integer.parseInt(parser.getAttributeValue(null, "dcl"));
            animation.default_deceleration = Integer.parseInt(parser.getAttributeValue(null, "def_dcl"));
            animation.flags = Integer.parseInt(parser.getAttributeValue(null, "flags"));
            if (animation.name != null && !"".equals(animation.name))
            {
                return animation;
            }
        }
        catch (Exception e)
        {
            Log.e("WC24h", "error reading xml file", e);
        }
        return null;
    }
};
